#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
mbS00.py
"""

import sys
import serial
from time import sleep
from crc16c import calcByte,calcString,hexCRC,checkModbusWithCrc,readHex
from threading import Timer
#from key_getter import KeyGetter # Tan sols funciona amb entorns amb GUI. Està basat en curses

class ModBus():
	def __init__(self, parent=None):
		
		if len( sys.argv ) == 2 :
			szPort = sys.argv[1]
			nBauds = 115200
		else :
			if len( sys.argv ) == 3 :
				szPort = sys.argv[1]
				nBauds = int(sys.argv[2])
			else :
				szPort = "/dev/ttyUSB0"
				nBauds = 4800

		port = szPort
		baudrate = nBauds
		
		print("Baudrate: %d at %s"%(baudrate,port))
		self.ser = 0
		self.t = 0
		
		try: 	
			self.ser = serial.Serial(
				port,
				baudrate,
				timeout=0,
				parity=serial.PARITY_NONE,
				stopbits=serial.STOPBITS_TWO,
				bytesize=serial.EIGHTBITS
			)
			
			self.nSegonsTimer = 0.2
			self.t = Timer(self.nSegonsTimer,self.temporitzador)
			self.t.start()
			
			#k = KeyGetter()
			self.bLoop = True
			while self.bLoop == True:
				#if k.kbhit():
				#	cKey = repr(k.getch(False))
				#	self.processaTecla(cKey[1])
				self.lecturaMB('noise')
				#sleep(0.1)
				sleep(2.0)
			self.bye()

		except serial.SerialException as e:
			print ("%s port is not connected" % szPort)
			sys.exit(-1)

		except KeyboardInterrupt:
			print ("\nAdéu!")
			self.t.cancel()		
			sys.exit(0)

	def writeCommand(self,command):
		hCRC = hexCRC(command)
		addedCRC="%c%c"%(hCRC>>8,hCRC&0xFF)
		szAddedCRC="%2X%2X"%(hCRC>>8,hCRC&0xFF)
		command += addedCRC
		#readHex(stMB)
		#print("")

		for cmd_byte in command:
			hex_byte = ("{0:02X}".format(ord(cmd_byte)))
			print (hex_byte,end='')
			self.ser.write(bytearray.fromhex(hex_byte))
		print("")		

	def lecturaMB(self,q):
		print("%s" % q)
		if q == 'noise':
			print("Reading noise")
			#01 03 00 00 00 01
			stMB = "%c%c%c%c%c%c"%(0x01,0x03,0x00,0x00,0x00,0x01)
			self.writeCommand(stMB)
			
	def processaTecla(self,c):
		if c == 'q' or c == 'Q':
			self.bLoop = False
			return
		if c == 'n':
			self.lecturaMB('noise')

	def processa(self,sz,nQ):
		#self.bConn = 0
		#print ("Rebut: %s"%sz)
		print ("He rebut %d bytes" % nQ)
		#readHex(sz)
		for b in sz:
			hex_byte = ("{0:02X}".format(b))
			print (hex_byte,end='')		
		print("")
		if nQ == 7:
			szByH = ("{0:02X}".format(sz[3]))
			szByL = ("{0:02X}".format(sz[4]))
			szWord = szByH + szByL
			nV = int(szWord,16)
			print("Dada: %s: %d"%(szWord,nV))

	def bye(self):
		if self.t:
			self.t.cancel()	
		if self.ser:
			self.ser.close()
		
	def __del__(self):
		self.bye()
					
	def temporitzador(self):
		data = self.ser.read(1)
		n = self.ser.inWaiting()
		if n:
			n = 1 + n
			data = data + self.ser.read(n)
			print("n: %d" % n)
		if len(data):
			print("Sí")
			self.processa(data,n)
			
		self.t = Timer(self.nSegonsTimer,self.temporitzador)
		self.t.start()
			
if __name__ == '__main__':

	mb = ModBus()

